import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import {
  csv,
  scaleBand,
  scaleLinear,
  scaleOrdinal,
  max,
  extent,
} from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';

const width = 960;
const height = 500;
const margin = {
  top: 20,
  right: 30,
  bottom: 65,
  left: 100,
};

const xAxisLabelOffset = 50;
const yAxisLabelOffset = 40;

const App = () => {
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }

  // Set margin
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  // assign values to avoid repeated logic
  const yValue = (d) => d.DiabetesPedigreeFunction;
  const xValue = (d) => d.Glucose;

  const xAxisLabel = 'Glucose';
  const yAxisLabel = 'DiabetesPedigreeFunction';
  
  const colorValue = d => d.Outcome;

  // turn the country names into y
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);

  // turn the population into x
  const xScale = scaleLinear()
    .domain(extent(data, xValue)) // extent = [min(data, xValue)->max(data,xValue)]
    .range([0, innerWidth]);
  
  const colorSet = ["#E6842A", "#137B80"];
  
  const colorScale = scaleOrdinal().domain(data.map(colorValue)).range(colorSet);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        // add tick lines
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          // Format the data
          tickFormat={(n) =>
            d3.format('.2s')(n).replace('G', 'B')
          }
        />
        <text
          className="axis-label"
          textAnchor="middle"
          transform={`translate(${-yAxisLabelOffset} ${
            innerHeight / 2
          }) rotate(-90) `}
        >
          {yAxisLabel}
        </text>
        <AxisLeft yScale={yScale} innerWidth={innerWidth} />
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset}
          textAnchor="middle"
        >
          {xAxisLabel}
        </text>
        <Marks
          data={data}
          xScale={xScale}
          yScale={yScale}
          colorScale = {colorScale}
          xValue={xValue}
          yValue={yValue}
          colorValue = {colorValue}
          tooltipFormat={(n) =>
            d3.format('.2s')(n).replace('G', 'B')
          }
          circleRadius={7}
        />
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
